@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 427 $ : Last Committed Revision
- * $Date: 2009-09-18 12:48:01 -0700 (Fri, 18 Sep 2009) $ : Last Committed Date */
+ * $Revision: 530 $ : Last Committed Revision
+ * $Date: 2010-06-16 16:52:19 -0700 (Wed, 16 Jun 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -292,8 +292,6 @@ acc_Thread( LPVOID arg )
 {
   connection_p connection;
   mc_platform_p mc_platform = (mc_platform_p)arg;
-  int err;
-  int i, j;
 
 	connection_thread_arg_t* connection_thread_arg;
 
@@ -672,8 +670,6 @@ listen_Thread( LPVOID arg )
 
 #endif
 
-    unsigned char aes_key[35];
-  int nonce, ret;
   char peer_name[45];	
   connection_p connection;
   u_long connection_number;
@@ -731,7 +727,7 @@ listen_Thread( LPVOID arg )
       {
         /* Acquire host name of connected   */
 #ifndef _WIN32
-        getnameinfo(&peer_addr, sizeof(peer_addr),
+        getnameinfo((const struct sockaddr*)&peer_addr, sizeof(peer_addr),
             peer_name, sizeof(peer_name), NULL, 0, 0);
 #else
         addr.s_addr = inet_addr( inet_ntoa(peer_addr.sin_addr) );
@@ -797,7 +793,7 @@ udplisten_Thread( LPVOID arg )
 {
   mc_platform_p mc_platform = (mc_platform_p)arg;
 	struct sockaddr_in si_me, si_remote;
-	int s, i, slen = sizeof(si_remote);
+	int s, slen = sizeof(si_remote);
 	char buf[BUFLEN];
 	char mc_req_string[] = "Mobile-C Agency Information Request";
 
