@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 402 $ : Last Committed Revision
- * $Date: 2009-08-23 13:44:39 -0700 (Sun, 23 Aug 2009) $ : Last Committed Date */
+ * $Revision: 517 $ : Last Committed Revision
+ * $Date: 2010-06-11 12:06:47 -0700 (Fri, 11 Jun 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -71,14 +71,14 @@ agent_datastate_Copy(const agent_datastate_p datastate)
   code_itr = datastate->agent_codes;
   i = 0;
   while (*code_id_itr != NULL && *code_itr != NULL) {
-    cp_data->agent_code_ids[i] = malloc
+    cp_data->agent_code_ids[i] = (char*)malloc
       (
        sizeof(char) * 
        (strlen(*code_id_itr) + 1)
       );
     strcpy(cp_data->agent_code_ids[i], *code_id_itr);
 
-    cp_data->agent_codes[i] = malloc
+    cp_data->agent_codes[i] = (char*)malloc
       (
        sizeof(char) * 
        (strlen(*code_itr) + 1 )
