@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 316 $ : Last Committed Revision
- * $Date: 2009-04-29 16:40:54 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 529 $ : Last Committed Revision
+ * $Date: 2010-06-16 16:00:50 -0700 (Wed, 16 Jun 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -93,6 +93,8 @@ df_Destroy(df_p df)
   COND_DESTROY(df->cond);
   free(df->lock);
   free(df->cond);
+  free(df->waiting_lock);
+  free(df->waiting_cond);
   free(df);
   return MC_SUCCESS;
 }
@@ -373,7 +375,7 @@ df_request_list_Pop(df_request_list_p requests)
     MUTEX_UNLOCK( requests->lock );
     return NULL;
   }
-  node = ListPop(requests->request_list);
+  node = (df_request_list_node_t*)ListPop(requests->request_list);
   requests->size--;
   MUTEX_UNLOCK( requests->lock );
   return node;
