@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 402 $ : Last Committed Revision
- * $Date: 2009-08-23 13:44:39 -0700 (Sun, 23 Aug 2009) $ : Last Committed Date */
+ * $Revision: 517 $ : Last Committed Revision
+ * $Date: 2010-06-11 12:06:47 -0700 (Fri, 11 Jun 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -84,7 +84,7 @@ name##_p name##_New( void ) \
 int name##_Destroy( name##_p name ) \
 { \
   struct node_type##_s* node; \
-  while ((node = name##_Pop(name)) != NULL) { \
+  while ((node = (node_type##_t*)name##_Pop(name)) != NULL) { \
     node_type##_Destroy(node); \
   } \
   ListTerminate(name->list); \
@@ -122,7 +122,7 @@ name##_p name##_Copy(name##_p name) \
   COND_INIT(temp->cond); \
   for( \
       i = 0; \
-      (temp_node = name##_SearchIndex(name, i)) != NULL; \
+      (temp_node = (node_type##_t*)name##_SearchIndex(name, i)) != NULL; \
       i++ \
      ) \
   { \
@@ -149,7 +149,7 @@ node_type##_t* name##_Pop( name##_p name ) \
     MUTEX_UNLOCK(name->lock); \
     return NULL; \
   } \
-  ret = ListPop(name->list); \
+  ret = (node_type##_t*)ListPop(name->list); \
   name->size--; \
   COND_SIGNAL(name->cond); \
   MUTEX_UNLOCK(name->lock); \
@@ -163,7 +163,7 @@ node_type##_t* name##_WaitPop( name##_p name ) \
   while (name->size <= 0) { \
 		COND_WAIT(name->cond, name->lock); \
   } \
-  ret = ListPop(name->list); \
+  ret = (node_type##_t*)ListPop(name->list); \
   name->size--; \
   COND_SIGNAL(name->cond); \
   MUTEX_UNLOCK(name->lock); \
@@ -183,7 +183,7 @@ int name##_RemoveIndex( name##_p name, int index ) \
 { \
   struct node_type##_s* node; \
   MUTEX_LOCK(name->lock); \
-  node = ListDelete(name->list, index); \
+  node = (node_type##_t*)ListDelete(name->list, index); \
   node_type##_Destroy(node); \
   name->size--; \
   MUTEX_UNLOCK(name->lock); \
