@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 402 $ : Last Committed Revision
- * $Date: 2009-08-23 13:44:39 -0700 (Sun, 23 Aug 2009) $ : Last Committed Date */
+ * $Revision: 517 $ : Last Committed Revision
+ * $Date: 2010-06-11 12:06:47 -0700 (Fri, 11 Jun 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -112,6 +112,8 @@ AP_QUEUE_GENERIC_DECL_TEMPLATE(
 
 int
 agent_queue_Print(agent_queue_p queue);
+int
+agent_queue_Flush(agent_queue_p queue);
 
 /* Agent variables */
 #include "interpreter_variable_data.h"
@@ -184,5 +186,7 @@ AP_QUEUE_DECL_TEMPLATE(
 		AP_GENERIC
 		)
 
+AP_GENERIC_p AP_GENERIC_Copy(AP_GENERIC_p AP_GENERIC);
+int AP_GENERIC_Destroy(AP_GENERIC_t* blah);
 
 #endif
