@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 447 $ : Last Committed Revision
- * $Date: 2010-03-17 13:30:32 -0700 (Wed, 17 Mar 2010) $ : Last Committed Date */
+ * $Revision: 519 $ : Last Committed Revision
+ * $Date: 2010-06-11 14:26:03 -0700 (Fri, 11 Jun 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -125,9 +125,12 @@ MC_WaitSignal
 #ifndef _MOBILEC_H_
 #define _MOBILEC_H_
 
-#ifndef THREAD_T
 #ifdef _WIN32
 #include <windows.h>
+#endif
+
+#ifndef THREAD_T
+#ifdef _WIN32
 #define THREAD_T HANDLE
 #else
 #define THREAD_T pthread_t
@@ -261,12 +264,13 @@ typedef struct MCAgencyOptions_s{
     ChOptions_t* ch_options;
 } MCAgencyOptions_t;
 
-typedef struct agent_thread_arg_s {
+typedef struct stationary_agent_info_s{
 	void* args;
 	struct agent_s* agent;
-	MCAgency_t attr;
+	MCAgency_t attr; /* DEPRECATED: Use the 'agency' member instead */
+	MCAgency_t agency;
 	THREAD_T thread;
-} agent_thread_arg_t;
+} stationary_agent_info_t;
 
 #ifndef AGENT_T
 #define AGENT_T
@@ -396,6 +400,26 @@ EXPORTMC extern struct fipa_acl_message_s* MC_AclWaitRetrieve(MCAgent_t agent);
 enum fipa_performative_e;
 enum fipa_protocol_e;
 
+EXPORTMC enum fipa_protocol_e MC_AclGetProtocol(
+    struct fipa_acl_message_s* acl);
+
+EXPORTMC char* MC_AclGetConversationID(
+    struct fipa_acl_message_s* acl);
+
+EXPORTMC enum fipa_performative_e MC_AclGetPerformative(
+    struct fipa_acl_message_s* acl);
+
+EXPORTMC int MC_AclGetSender(
+    struct fipa_acl_message_s* acl,
+    char** name, /* OUT: Will allocate a text string to be freed by the
+                          user. */
+    char** address /* OUT: Will allocate a text string to be freed by the user
+                    */
+    );
+  
+EXPORTMC const char* MC_AclGetContent(
+    struct fipa_acl_message_s* acl);
+
 EXPORTMC int MC_AclSetProtocol(
     struct fipa_acl_message_s* acl,
     enum fipa_protocol_e performative );
@@ -443,11 +467,175 @@ EXPORTMC extern int MC_AddAgent(
     MCAgent_t agent);
 
 
-#ifndef _WIN32
-int MC_AddStationaryAgent(MCAgency_t agency, void* (*agent_thread)(struct agent_thread_arg_s*), const char* name, void* agent_args);
-#else
-int MC_AddStationaryAgent(MCAgency_t agency, LPTHREAD_START_ROUTINE agent_thread, const char* name, void* agent_args);
-#endif
+int MC_AddStationaryAgent(
+    MCAgency_t agency, 
+    void* (*agent_thread)(stationary_agent_info_t*), 
+    const char* name, 
+    void* agent_args);
+
+
+/**
+ * \brief           Add a new task to an already existing agent
+ *                  
+ * \param agent     An existing agent
+ * \param code      The C code to use in the new task
+ * \param return_var_name The name of the the return variable, or NULL if there
+ *                  is no return variable.
+ * \param server    The server to perform the task on.
+ * \param persistent Whether or not the task is persistent. '1' indicates
+ *                  persistent, and '0' indicates not-persistent (the default)
+ * \return          Returns a valid agency handle or NULL on failure.
+ */
+int MC_AgentAddTask(
+    MCAgent_t agent, 
+    const char* code, 
+    const char* return_var_name, 
+    const char* server,
+    int persistent);
+
+/**
+ * \brief           Add a new task to an already existing agent
+ *                  
+ * \param agent     An existing agent
+ * \param filename  The C code to use in the new task
+ * \param return_var_name The name of the the return variable, or NULL if there
+ *                  is no return variable.
+ * \param server    The server to perform the task on.
+ * \param persistent Whether or not the task is persistent. '1' indicates
+ *                  persistent, and '0' indicates not-persistent (the default)
+ * \return          Returns a valid agency handle or NULL on failure.
+ */
+int MC_AgentAddTaskFromFile(
+    MCAgent_t agent, 
+    const char* filename, 
+    const char* return_var_name, 
+    const char* server,
+    int persistent);
+
+/**
+ * \brief           Get the host Mobile-C agency handle from a
+ *                  "stationary_agent_info_t" data structure.
+ *                  
+ * \param stationary_agent_info A stationary_agent_info_t type variable
+ *                  provided to a stationary agent function
+ * \return          Returns a valid agency handle or NULL on failure.
+ */
+extern MCAgency_t 
+MC_AgentInfo_GetAgency(stationary_agent_info_t* stationary_agent_info);
+
+/**
+ * \brief           Retrieve an agent handle from the stationary agent
+ *                  info from within a stationary agent.
+ * \param stationary_agent_info A stationary_agent_info_t type variable
+ *                  provided to a stationary agent function
+ * \return          Returns a valid agent handle or NULL on failure.
+ * \section         Examples
+ * The following example demonstrates usage of MC_AgentInfo_GetAgent()
+ * \include stationary_agent_communication/server.c
+ */
+extern MCAgent_t 
+MC_AgentInfo_GetAgent(stationary_agent_info_t* stationary_agent_info);
+
+/**
+ * \brief           Retrieve the agent argument supplied as the "agent_args"
+ *                  parameter in the MC_AddStationaryAgent() function.
+ *                  
+ * \param stationary_agent_info A stationary_agent_info_t type variable
+ *                  provided to a stationary agent function
+ * \return          Returns a valid pointer or NULL on failure.
+ */
+extern void*
+MC_AgentInfo_GetAgentArgs(stationary_agent_info_t* stationary_agent_info);
+
+/**
+ * \brief           Get the dimension of an agent's return data, if it is an array
+ *
+ * \param agent     A return agent
+ * \param task      task to retrieve the agent return data size from
+ *
+ * \return          Returns a positive integer on success or -1 on failure.
+ */
+extern int MC_AgentReturnArrayDim(
+    MCAgent_t agent,
+    int task_num );
+
+/**
+ * \brief           Get the extent of a dimension of an agent's return data, if it is an array
+ *
+ * \param agent     A return agent
+ * \param task      task to retrieve the agent return data size from
+ * \param index     The dimension index to get the extent
+ *
+ * \return          Returns a positive integer on success or -1 on failure.
+ */
+extern int MC_AgentReturnArrayExtent(
+    MCAgent_t agent,
+    int task_num,
+    int index);
+
+/**
+ * \brief           Get the total number of elements of the agent return data.
+ *
+ * \param agent     A return agent
+ * \param task      task to retrieve the agent return data size from
+ * \param index     The dimension index to get the extent
+ *
+ * \return          Returns a positive integer on success or -1 on failure.
+ */
+extern int MC_AgentReturnArrayNum(
+    MCAgent_t agent,
+    int task_num);
+
+/**
+ * \brief           Get a pointer to the agent's return data
+ *
+ * \param agent     A return agent
+ * \param task      task to retrieve the agent return data from
+ *
+ * \return          Returns a pointer to the beginning of the task's return
+ *                  data on success. Returns NULL on failure.
+ */
+extern const void* MC_AgentReturnDataGetSymbolAddr(
+    MCAgent_t agent,
+    int task_num );
+
+/**
+ * \brief           Get the size of an agent's return data type. 
+ *
+ * \param agent     A return agent
+ * \param task      task to retrieve the agent return data type size from
+ *
+ * \return          Returns a positive integer on success or -1 on failure.
+ */
+extern size_t MC_AgentReturnDataSize(
+    MCAgent_t agent,
+    int task_num );
+
+/**
+ * \brief           Get the type of an agent's return data
+ *
+ * \param agent     A return agent
+ * \param task      task to retrieve the agent return data type from
+ *
+ * \return          Returns a positive integer on success or -1 on failure.
+ *                  The return type is of type ChType_t, defined in ch.h
+ */
+extern int MC_AgentReturnDataType(
+    MCAgent_t agent,
+    int task_num );
+
+/**
+ * \brief           Check to see whether or not an agent's return data is an array.
+ *
+ * \param agent     A return agent
+ * \param task      task to check to see if the return data is an array.
+ *
+ * \return          Returns 1 if the return variable is an array, 0 if it is
+ *                  not an array, or -1 on failure.
+ */
+extern int MC_AgentReturnIsArray(
+    MCAgent_t agent,
+    int task_num );
 
 /**
  * \brief           Retrieve a pointer to a previously saved variable.
@@ -514,6 +702,9 @@ MC_AgentVariableRetrieveInfo(
 extern int 
 MC_AgentVariableSave(MCAgent_t agent, const char* var_name);
 
+EXPORTMC int 
+MC_Barrier(MCAgency_t attr, int id); 
+
 /**
  * \brief           Find and delete an initialized MobileC Barrier
  *
@@ -670,6 +861,8 @@ MC_ComposeAgent(
  * \param persitent A flag indicating whether or not the agent should be persistent. A value of '1' indicates persistence, while a value of '0' indicates default non-persistent behaviour.
  *
  * \return          This function returns a valid MCAgent_t structure on success or NULL on failure.
+ * \note            This function is deprecated. Please use the
+ *                  MC_ComposeAgentWithWorkgroup function instead.
  */
 EXPORTMC extern MCAgent_t
 MC_ComposeAgentS(
@@ -683,6 +876,33 @@ MC_ComposeAgentS(
 		const char* workgroup_code
     );
 
+/**
+ * \brief           Compose a new agent dynamically without using a prewritten XML file.
+ *
+ * \param name      The desired name of the new agent.
+ * \param home      The home of the new agent.
+ * \param owner     The owner of the new agent.
+ * \param code      The agent code
+ * \param return_var_name The name of the agent's return variable. Set to "no-return" if no return variable is desired.
+ * \param server    The target destination of the agent.
+ * \param workgroup_code The secret workgroup code of the agent. Only agents with the same workgroup code may perform certain interactions.
+ * \param persitent A flag indicating whether or not the agent should be persistent. A value of '1' indicates persistence, while a value of '0' indicates default non-persistent behaviour.
+ *
+ * \return          This function returns a valid MCAgent_t structure on success or NULL on failure.
+ * \note            This function is deprecated
+ */
+EXPORTMC extern MCAgent_t
+MC_ComposeAgentWithWorkgroup(
+    const char* name,
+    const char* home,
+    const char* owner, 
+    const char* code,
+    const char* return_var_name,
+    const char* server,
+    int persistent,
+		const char* workgroup_code
+    );
+
 /**
  * \brief           Compose a new agent dynamically from a source code file
  *
@@ -722,6 +942,8 @@ MC_ComposeAgentFromFile(
  * \param persitent A flag indicating whether or not the agent should be persistent. A value of '1' indicates persistence, while a value of '0' indicates default non-persistent behaviour.
  *
  * \return          This function returns a valid MCAgent_t structure on success or NULL on failure.
+ * \note            This function is deprecated. Please use the
+ *                  MC_ComposeAgentFromFileWithWorkgroup function instead.
  */
 EXPORTMC extern MCAgent_t
 MC_ComposeAgentFromFileS(
@@ -735,6 +957,33 @@ MC_ComposeAgentFromFileS(
 		const char* workgroup_code
     );
 
+/**
+ * \brief           Compose a new agent dynamically from a source code file
+ *
+ * \param filename  The filename of the source file
+ * \param name      The desired name of the new agent.
+ * \param home      The home of the new agent.
+ * \param owner     The owner of the new agent.
+ * \param code      The agent code
+ * \param return_var_name The name of the agent's return variable. Set to "no-return" if no return variable is desired.
+ * \param server    The target destination of the agent.
+ * \param workgroup_code The secret workgroup code of the agent. Only agents with the same workgroup code may perform certain interactions.
+ * \param persitent A flag indicating whether or not the agent should be persistent. A value of '1' indicates persistence, while a value of '0' indicates default non-persistent behaviour.
+ *
+ * \return          This function returns a valid MCAgent_t structure on success or NULL on failure.
+ */
+EXPORTMC extern MCAgent_t
+MC_ComposeAgentFromFileWithWorkgroup(
+    const char* name,
+    const char* home,
+    const char* owner, 
+    const char* filename,
+    const char* return_var_name,
+    const char* server,
+    int persistent,
+		const char* workgroup_code
+    );
+
 /**
  * \brief           Wakes up all agents/threads waiting on a condition variable
  *
@@ -1225,6 +1474,24 @@ EXPORTMC extern int MC_SearchForService(
         int** agentIDs,
         int* numResults);
 
+/**
+ * \brief           Sends an agent migration message
+ *
+ * \param attr      a MobileC agency handle
+ * \param message   a valid MobileC xml agent migration message
+ */
+EXPORTMC extern int MC_SendAgent(MCAgency_t attr,
+        const char *message);
+
+/**
+ * \brief           Sends an agent migration message
+ *
+ * \param attr      a MobileC agency handle
+ * \param filename  file containing a valid MobileC xml agent migration message
+ */
+EXPORTMC extern int MC_SendAgentFile(MCAgency_t attr, 
+        const char *filename);
+
 /**
  * \brief           Sends an agent migration message
  *
@@ -1259,7 +1526,7 @@ EXPORTMC extern int MC_SendAgentMigrationMessageFile(MCAgency_t attr,
  *
  * \return          0 on success, or error_code_t on failure
  */
-EXPORTMC extern int MC_SetAgentStatus(MCAgent_t agent, int status);
+EXPORTMC extern int MC_SetAgentStatus(MCAgent_t agent, enum MC_AgentStatus_e status);
 
 /**
  * \brief           Set up a steerable algorithm
