@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 468 $ : Last Committed Revision
- * $Date: 2010-05-17 12:52:54 -0700 (Mon, 17 May 2010) $ : Last Committed Date */
+ * $Revision: 522 $ : Last Committed Revision
+ * $Date: 2010-06-11 16:52:39 -0700 (Fri, 11 Jun 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -63,6 +63,7 @@
 #include "include/fipa_acl.h"
 #include "include/agent.h"
 #include "include/agent_task.h"
+#include "include/agent_lib.h"
 #ifndef HOST_NAME_MAX
 #define HOST_NAME_MAX 255
 #endif
@@ -74,7 +75,7 @@ mc_platform_p g_mc_platform;
  * Libmc Binary Space Functions *
  * **************************** */
 
-int 
+EXPORTMC int 
 MC_AclDestroy(struct fipa_acl_message_s* message)
 {
   return fipa_acl_message_Destroy(message);
@@ -209,7 +210,44 @@ MC_AclWaitRetrieve(MCAgent_t agent)
 
 /* ACL Helper Functions Here */
 
-int MC_AclSetProtocol(
+EXPORTMC enum fipa_protocol_e MC_AclGetProtocol(
+    struct fipa_acl_message_s* acl)
+{
+  return (enum fipa_protocol_e)acl->performative;
+}
+
+EXPORTMC char* MC_AclGetConversationID(
+    struct fipa_acl_message_s* acl)
+{
+  return acl->conversation_id->content.string->content;
+}
+
+EXPORTMC enum fipa_performative_e MC_AclGetPerformative(
+    struct fipa_acl_message_s* acl)
+{
+  return acl->performative;
+}
+
+EXPORTMC int MC_AclGetSender(
+    struct fipa_acl_message_s* acl,
+    char** name, /* OUT: Will allocate a text string to be freed by the
+                          user. */
+    char** address /* OUT: Will allocate a text string to be freed by the user
+                    */
+    )
+{
+  *name = strdup(acl->sender->name);
+  *address = strdup( acl->sender->addresses->urls[0]->str );
+  return 0;
+}
+
+EXPORTMC const char* MC_AclGetContent(
+    struct fipa_acl_message_s* acl)
+{
+  return acl->content->content;
+}
+
+EXPORTMC int MC_AclSetProtocol(
     fipa_acl_message_t* acl,
     enum fipa_protocol_e protocol)
 {
@@ -225,7 +263,7 @@ int MC_AclSetProtocol(
   return 0;
 }
 
-int MC_AclSetConversationID(
+EXPORTMC int MC_AclSetConversationID(
     fipa_acl_message_t* acl,
     char* id)
 {
@@ -244,7 +282,7 @@ int MC_AclSetConversationID(
   return 0;
 }
 
-int MC_AclSetPerformative(
+EXPORTMC int MC_AclSetPerformative(
     fipa_acl_message_t* acl,
     enum fipa_performative_e performative )
 {
@@ -252,7 +290,7 @@ int MC_AclSetPerformative(
   return 0;
 }
 
-int MC_AclSetSender(
+EXPORTMC int MC_AclSetSender(
     fipa_acl_message_t* acl,
     const char* name,
     const char* address )
@@ -275,7 +313,7 @@ int MC_AclSetSender(
   return 0;
 }
   
-int MC_AclAddReceiver(
+EXPORTMC int MC_AclAddReceiver(
     fipa_acl_message_t* acl,
     const char* name,
     const char* address )
@@ -311,7 +349,7 @@ int MC_AclAddReceiver(
   return 0;
 }
 
-int MC_AclAddReplyTo(
+EXPORTMC int MC_AclAddReplyTo(
     fipa_acl_message_t* acl,
     const char* name,
     const char* address)
@@ -346,7 +384,7 @@ int MC_AclAddReplyTo(
   return 0;
 }
 
-int MC_AclSetContent(
+EXPORTMC int MC_AclSetContent(
     fipa_acl_message_t* acl,
     const char* content )
 {
@@ -376,11 +414,11 @@ MC_AddAgent(MCAgency_t attr, MCAgent_t agent) /*{{{*/
   return 0;
 } /*}}}*/
 
-#ifndef _WIN32
-int MC_AddStationaryAgent(MCAgency_t agency, void* (*agent_thread)(struct agent_thread_arg_s*), const char* name, void* agent_args)
-#else
-int MC_AddStationaryAgent(MCAgency_t agency, LPTHREAD_START_ROUTINE agent_thread, const char* name, void* agent_args)
-#endif
+EXPORTMC int MC_AddStationaryAgent(
+    MCAgency_t agency, 
+    void* (*agent_thread)(stationary_agent_info_t*), 
+    const char* name, 
+    void* agent_args)
 {
 #ifndef _WIN32
 	pthread_attr_t attr;
@@ -388,17 +426,288 @@ int MC_AddStationaryAgent(MCAgency_t agency, LPTHREAD_START_ROUTINE agent_thread
 #else
 	int stack_size = 0;
 #endif
-	struct agent_thread_arg_s* agent_thread_arg;
-	agent_thread_arg = (agent_thread_arg_t*)malloc(sizeof(agent_thread_arg_t));
-	agent_thread_arg->args = agent_args;
-	agent_thread_arg->agent = agent_NewBinary(agency->mc_platform);
-	agent_thread_arg->agent->name = strdup(name);
-	agent_thread_arg->attr = agency;
-	agent_queue_Add(agency->mc_platform->agent_queue, agent_thread_arg->agent);
-	THREAD_CREATE(&agent_thread_arg->thread, agent_thread, agent_thread_arg);
+	stationary_agent_info_t* stationary_agent_info;
+	stationary_agent_info = (stationary_agent_info_t*)malloc(sizeof(stationary_agent_info_t));
+	stationary_agent_info->args = agent_args;
+	stationary_agent_info->agent = agent_NewBinary(agency->mc_platform);
+	stationary_agent_info->agent->name = strdup(name);
+	stationary_agent_info->attr = agency;
+	stationary_agent_info->agency = agency;
+	agent_queue_Add(agency->mc_platform->agent_queue, stationary_agent_info->agent);
+#ifndef _WIN32
+	THREAD_CREATE(&stationary_agent_info->thread, agent_thread, stationary_agent_info);
+#else
+	THREAD_CREATE(&stationary_agent_info->thread, (LPTHREAD_START_ROUTINE)agent_thread, stationary_agent_info);
+#endif
+  return 0;
 }
 
-const void* MC_AgentVariableRetrieve(MCAgent_t agent, const char* var_name, int task_num)
+EXPORTMC MCAgency_t MC_AgentInfo_GetAgency(stationary_agent_info_t* stationary_agent_info)
+{
+  return stationary_agent_info->agency;
+}
+
+EXPORTMC MCAgent_t MC_AgentInfo_GetAgent(stationary_agent_info_t* stationary_agent_info)
+{
+  return stationary_agent_info->agent;
+}
+
+EXPORTMC void* MC_AgentInfo_GetAgentArgs(stationary_agent_info_t* stationary_agent_info)
+{
+  return stationary_agent_info->args;
+}
+
+EXPORTMC int MC_AgentAddTask(
+    MCAgent_t agent, 
+    const char* code, 
+    const char* return_var_name, 
+    const char* server,
+    int persistent)
+{
+  int new_num_tasks;
+  int num_code_ids;
+  int i, j;
+  agent->datastate->number_of_tasks++;
+  new_num_tasks = agent->datastate->number_of_tasks;
+
+  /* Allocate new code id */
+  /* First, we need to find the number of agent_code_ids */
+  for(num_code_ids = 0; 
+      agent->datastate->agent_code_ids[num_code_ids] != NULL; 
+      num_code_ids++);
+
+  agent->datastate->agent_code_ids = (char**)realloc(
+      agent->datastate->agent_code_ids,
+      sizeof(char*)*(num_code_ids + 2));
+  agent->datastate->agent_code_ids[num_code_ids] = NULL;
+  agent->datastate->agent_code_ids[num_code_ids+1] = NULL;
+  if(agent->datastate->agent_code_ids == NULL) {
+    fprintf(stderr, "Memory Error %s:%d\n", __FILE__, __LINE__);
+  }
+
+  /* If there is an agent code without an ID, then we must find all tasks
+   * without an agent code and assign the un-id'd code to those tasks. */
+  for(i = 0; agent->datastate->agent_code_ids[i] != NULL; i++) {
+    if(strlen(agent->datastate->agent_code_ids[i]) == 0) {
+      free(agent->datastate->agent_code_ids[i]);
+      agent->datastate->agent_code_ids[i] = (char*)malloc(sizeof(char)*20);
+      sprintf(agent->datastate->agent_code_ids[i], "code%d", rand());
+      for(j = 0; j < agent->datastate->number_of_tasks-1; j++) {
+        if (agent->datastate->tasks[j]->code_id == NULL) {
+          agent->datastate->tasks[j]->code_id = strdup(agent->datastate->agent_code_ids[i]);
+        }
+      }
+      break;
+    }
+  }
+
+  /* Make a new code id for the new task */
+  agent->datastate->agent_code_ids[num_code_ids] = (char*)malloc(sizeof(char)*20);
+  sprintf(agent->datastate->agent_code_ids[num_code_ids], "code%d", rand());
+
+  /* Allocate new agent_code */
+  agent->datastate->agent_codes = (char**)realloc(
+      agent->datastate->agent_codes,
+      sizeof(char*)*(num_code_ids + 2));
+  agent->datastate->agent_codes[num_code_ids] = NULL;
+  agent->datastate->agent_codes[num_code_ids+1] = NULL;
+  agent->datastate->agent_codes[num_code_ids] = strdup(code);
+
+  /* Allocate new task */
+  agent->datastate->tasks = (agent_task_t**)realloc(
+      agent->datastate->tasks,
+      sizeof(agent_task_t*) * new_num_tasks );
+  agent->datastate->tasks[new_num_tasks-1] = agent_task_New();
+  if(return_var_name == NULL) {
+    agent->datastate->tasks[new_num_tasks-1]->var_name = strdup("no-return");
+  } else {
+    agent->datastate->tasks[new_num_tasks-1]->var_name = strdup(return_var_name);
+  }
+  agent->datastate->tasks[new_num_tasks-1]->server_name = strdup(server);
+  agent->datastate->tasks[new_num_tasks-1]->persistent = persistent;
+  agent->datastate->tasks[new_num_tasks-1]->code_id = 
+      strdup(agent->datastate->agent_code_ids[num_code_ids]);
+  return 0;
+}
+
+EXPORTMC int MC_AgentAddTaskFromFile(
+    MCAgent_t agent, 
+    const char* filename, 
+    const char* return_var_name, 
+    const char* server,
+    int persistent)
+{
+  struct stat buffer;
+  char *code;
+  int codesize;
+  FILE* fptr;
+  int return_val;
+  if(stat(filename, &buffer)) {
+    /* Stat failed. Maybe file does not exist? */
+    fprintf(stderr, "MC_AgentAddTaskFromFile failed. File %s does not exist.\n",
+        filename);
+    return -1;
+  }
+  codesize = buffer.st_size;
+  code = (char *)malloc((codesize+1)*sizeof(char));
+  memset(code, 0, codesize+1);
+  fptr = fopen(filename, "r");
+  fread(code, sizeof(char), codesize, fptr);
+  fclose(fptr);
+  return_val = MC_AgentAddTask(
+      agent,
+      code,
+      return_var_name,
+      server,
+      persistent );
+  free(code);
+  return return_val;
+}
+
+EXPORTMC int MC_AgentReturnArrayExtent(
+    MCAgent_t agent,
+    int task_num,
+    int index )
+{
+  if (task_num >= agent->datastate->number_of_tasks) {
+    return -1;
+  }
+
+  if (
+      agent->datastate->tasks[task_num]->
+      agent_return_data->data_type == -1
+     )
+  {
+    return -1;
+  }
+
+  if (index >= agent->datastate->tasks[task_num]->agent_return_data->array_dim) {
+    return -1;
+  }
+
+  return agent->datastate->tasks[task_num]->agent_return_data->array_extent[index];
+}
+
+EXPORTMC int MC_AgentReturnArrayDim(
+    MCAgent_t agent,
+    int task_num )
+{
+  if (task_num >= agent->datastate->number_of_tasks) {
+    return -1;
+  }
+
+  if (
+      agent->datastate->tasks[task_num]->
+      agent_return_data->data_type == -1
+     )
+  {
+    return -1;
+  }
+  return agent->datastate->tasks[task_num]->agent_return_data->array_dim;
+}
+
+EXPORTMC int MC_AgentReturnArrayNum(
+    MCAgent_t agent,
+    int task_num )
+{
+  int num_elements, i;
+  if (task_num >= agent->datastate->number_of_tasks) {
+    return -1;
+  }
+
+  if (
+      agent->datastate->tasks[task_num]->
+      agent_return_data->data_type == -1
+     )
+  {
+    return -1;
+  }
+
+  num_elements = 1;
+  for (
+      i = 0; 
+      i < agent->datastate->tasks[task_num]->agent_return_data->array_dim;
+      i++
+      )
+  {
+    num_elements *= agent->datastate->
+      tasks[task_num]->agent_return_data->array_extent[i];
+  }
+  return num_elements;
+}
+
+EXPORTMC const void* MC_AgentReturnDataGetSymbolAddr(
+    MCAgent_t agent,
+    int task_num )
+{
+  if (task_num >= agent->datastate->number_of_tasks) {
+    return NULL;
+  }
+  if (
+      agent->datastate->tasks[task_num]->
+      agent_return_data->data_type == -1
+     )
+  {
+    return NULL;
+  }
+  return agent->datastate->tasks[task_num]->agent_return_data->data;
+}
+
+EXPORTMC size_t MC_AgentReturnDataSize(
+    MCAgent_t agent,
+    int task_num )
+{
+  int size;
+  if (task_num >= agent->datastate->number_of_tasks) {
+    return -1;
+  }
+
+  if (
+      agent->datastate->tasks[task_num]->
+      agent_return_data->data_type == -1
+     )
+  {
+    return -1;
+  }
+
+  CH_DATATYPE_SIZE(
+      agent->datastate->tasks[task_num]->agent_return_data->data_type,
+      size);
+  return size;
+}
+
+EXPORTMC int MC_AgentReturnDataType(
+    MCAgent_t agent,
+    int task_num )
+{
+  if (task_num >= agent->datastate->number_of_tasks) {
+    return -1;
+  }
+  return agent->datastate->tasks[task_num]->agent_return_data->data_type;
+}
+
+EXPORTMC int MC_AgentReturnIsArray(
+    MCAgent_t agent,
+    int task_num )
+{
+  if (task_num >= agent->datastate->number_of_tasks) {
+    return -1;
+  }
+  if (
+      agent->datastate->tasks[task_num]->
+      agent_return_data->data_type == -1
+     )
+  {
+    return -1;
+  }
+  if(agent->datastate->tasks[task_num]->agent_return_data->array_dim) {
+    return 1;
+  } else {
+    return 0;
+  }
+}
+
+EXPORTMC const void* MC_AgentVariableRetrieve(MCAgent_t agent, const char* var_name, int task_num)
 {
   interpreter_variable_data_t* interp_var;
 
@@ -416,7 +725,7 @@ const void* MC_AgentVariableRetrieve(MCAgent_t agent, const char* var_name, int
   return interp_var->data;
 }
 
-int MC_AgentVariableRetrieveInfo(MCAgent_t agent, const char* var_name, int task_num, const void** data, int* dim, const int** extent)
+EXPORTMC int MC_AgentVariableRetrieveInfo(MCAgent_t agent, const char* var_name, int task_num, const void** data, int* dim, const int** extent)
 {
   interpreter_variable_data_t* interp_var;
 
@@ -437,7 +746,7 @@ int MC_AgentVariableRetrieveInfo(MCAgent_t agent, const char* var_name, int task
   return MC_SUCCESS;
 }
 
-int MC_AgentVariableSave(MCAgent_t agent, const char* var_name)
+EXPORTMC int MC_AgentVariableSave(MCAgent_t agent, const char* var_name)
 {
   int current_task = agent->datastate->task_progress;
   const int default_num_vars = 50;
@@ -457,7 +766,7 @@ int MC_AgentVariableSave(MCAgent_t agent, const char* var_name)
   return 0;
 }
 
-int 
+EXPORTMC int 
 MC_Barrier(MCAgency_t attr, int id) /*{{{*/
 {
     barrier_queue_p list = attr->mc_platform->barrier_queue;
@@ -614,7 +923,7 @@ MC_ComposeAgent(
     int persistent
     )
 {
-  return  MC_ComposeAgentS(
+  return  MC_ComposeAgentWithWorkgroup(
       name,
       home,
       owner,
@@ -637,6 +946,32 @@ MC_ComposeAgentS(
     int persistent,
 		const char* workgroup_code
     )
+{
+  printf("\nThe MC_ComposeAgentS function is now deprecated due to its\n\
+  misleading name. Please use the MC_ComposeAgentWithWorkgroup() function\n\
+  instead.\n");
+  return MC_ComposeAgentWithWorkgroup(
+      name,
+      home,
+      owner,
+      code,
+      return_var_name,
+      server,
+      persistent,
+      workgroup_code );
+}
+
+EXPORTMC MCAgent_t
+MC_ComposeAgentWithWorkgroup(
+    const char* name,
+    const char* home,
+    const char* owner, 
+    const char* code,
+    const char* return_var_name,
+    const char* server,
+    int persistent,
+		const char* workgroup_code
+    )
 {
   agent_p agent;
   agent = agent_New();
@@ -652,7 +987,6 @@ MC_ComposeAgentS(
 
   agent->datastate = agent_datastate_New();
   agent->datastate->number_of_tasks = 1;
-  agent->datastate->persistent = persistent;
   agent->datastate->agent_code_ids = (char**)malloc(
       sizeof(char*)*2);
   if(agent->datastate->agent_code_ids == NULL) {
@@ -700,6 +1034,7 @@ MC_ComposeAgentS(
 			fprintf(stderr, "Memory Error %s:%d\n", __FILE__, __LINE__);
 		}
 	}
+  agent->datastate->tasks[0]->persistent = persistent;
  
   return agent;
 }
@@ -714,7 +1049,7 @@ EXPORTMC MCAgent_t MC_ComposeAgentFromFile(
     int persistent
     )
 {
-  return MC_ComposeAgentFromFileS(
+  return MC_ComposeAgentFromFileWithWorkgroup(
       name,
       home,
       owner,
@@ -736,6 +1071,31 @@ EXPORTMC MCAgent_t MC_ComposeAgentFromFileS(
     int persistent,
 		const char* workgroup_code
     )
+{
+  printf("\nThe MC_ComposeAgentFromFileS() function is deprecated due to \n\
+  its misleading name. Please use the MC_ComposeAgentFromFileWithWorkgroup\n\
+  function instead.\n");
+  return MC_ComposeAgentFromFileWithWorkgroup(
+      name,
+      home,
+      owner,
+      filename,
+      return_var_name,
+      server,
+      persistent,
+      workgroup_code);
+}
+
+EXPORTMC MCAgent_t MC_ComposeAgentFromFileWithWorkgroup(
+    const char* name,
+    const char* home,
+    const char* owner, 
+    const char* filename,
+    const char* return_var_name,
+    const char* server,
+    int persistent,
+		const char* workgroup_code
+    )
 {
   MCAgent_t agent;
   struct stat buffer;
@@ -754,7 +1114,7 @@ EXPORTMC MCAgent_t MC_ComposeAgentFromFileS(
   fread(code, sizeof(char), codesize, fptr);
   fclose(fptr);
 
-  agent = MC_ComposeAgentS(
+  agent = MC_ComposeAgentWithWorkgroup(
      name, 
      home, 
      owner, 
@@ -764,6 +1124,7 @@ EXPORTMC MCAgent_t MC_ComposeAgentFromFileS(
      persistent,
 		 workgroup_code
      );
+  free(code);
 
   return agent;
 }
@@ -838,7 +1199,7 @@ MC_CondReset(MCAgency_t attr, int id) /*{{{*/
     return 1;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_CopyAgent(MCAgent_t* agent_out, const MCAgent_t agent_in) /*{{{*/
 {
   *agent_out = agent_Copy(agent_in);
@@ -851,7 +1212,7 @@ MC_DeleteAgent(MCAgent_t agent) /*{{{*/
     /* Error Checking */
     CHECK_NULL(agent, return MC_ERR_INVALID;);
 
-    /* If the agent has a workgroup code, when the user must use MC_DeleteAgentWG() */
+    /* If the agent has a workgroup code, then the user must use MC_DeleteAgentWG() */
     if ( agent->wg_code != NULL ) {
       return MC_ERR_INVALID_ARGS;
     }
@@ -859,9 +1220,6 @@ MC_DeleteAgent(MCAgent_t agent) /*{{{*/
     /* First, make sure the agent is no longer running */
     MC_TerminateAgent(agent);
 
-    /* Now, we just set it's status to MC_WAIT_FINISHED so that the AMS will
-     * flush it next chance it gets. */
-    MC_SetAgentStatus(agent, MC_WAIT_FINISHED);
     return MC_SUCCESS;
 } /*}}}*/
 
@@ -883,13 +1241,10 @@ MC_DeleteAgentWG(MCAgent_t calling_agent, MCAgent_t agent) /*{{{*/
     /* First, make sure the agent is no longer running */
     MC_TerminateAgentWG(calling_agent, agent);
 
-    /* Now, we just set it's status to MC_WAIT_FINISHED so that the AMS will
-     * flush it next chance it gets. */
-    MC_SetAgentStatus(agent, MC_WAIT_FINISHED);
     return MC_SUCCESS;
 } /*}}}*/
 
-int MC_DestroyServiceSearchResult(
+EXPORTMC int MC_DestroyServiceSearchResult(
     char** agentName,
     char** serviceName,
     int* agentID,
@@ -908,7 +1263,7 @@ int MC_DestroyServiceSearchResult(
   return 0;
 }
 
-int 
+EXPORTMC int 
 MC_DeregisterService( /*{{{*/
         MCAgency_t agency,
         int agentID,
@@ -1026,6 +1381,7 @@ MC_FindAgentByID( MCAgency_t attr, /*{{{*/
   }
 } /*}}}*/
 
+EXPORTMC 
 #ifndef _WIN32
   time_t 
 #else
@@ -1110,6 +1466,8 @@ MC_GetAgentReturnData( /*{{{*/
   int num_elements;
   int size;
   int i;
+  printf("\nThe function MC_GetAgentReturnData() is deprecated.\n\
+  Please use the MC_AgentReturn*() series of functions instead.\n");
   if (task_num >= agent->datastate->number_of_tasks) {
     *data = NULL;
     *dim = 0;
@@ -1174,11 +1532,11 @@ MC_GetAgentType(MCAgent_t agent) /*{{{*/
   if (agent != NULL) {
     return agent->agent_type;
   } else {
-    return 0;
+    return (enum MC_AgentType_e)0;
   }
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_GetAllAgents(MCAgency_t attr, MCAgent_t **agents, int* num_agents) /*{{{*/
 {
   int halt;
@@ -1232,18 +1590,16 @@ MC_Initialize( /*{{{*/
   int i=0;
   int options_malloc = 0;
   char privkey[1210];   // private key after decryption
-  FILE *f;
-  char buf[4], ch;
 
   memset(privkey, '\0', 1210);
   ret = (MCAgency_t)malloc(sizeof(struct agency_s));
   if (ret == NULL) {return NULL;}
 
-  ret->hostName = malloc(HOST_NAME_MAX);
+  ret->hostName = (char*)malloc(HOST_NAME_MAX);
   if (ret->hostName == NULL) {return NULL;}
   gethostname(ret->hostName, HOST_NAME_MAX);
   /*Save some memory */
-  ret->hostName = realloc(ret->hostName, sizeof(char)*(strlen(ret->hostName)+1));
+  ret->hostName = (char*)realloc(ret->hostName, sizeof(char)*(strlen(ret->hostName)+1));
   if (ret->hostName == NULL) {return NULL;}
   ret->portno = port;
   ret->server = 1;
@@ -1364,7 +1720,7 @@ MC_LoadAgentFromFile(MCAgency_t attr, const char* filename)
   filestat.st_size = 0;
   stat(filename, &filestat);
   if (filestat.st_size != 0 ) {
-    buf = malloc( sizeof(char) * (filestat.st_size+1) );
+    buf = (char*)malloc( sizeof(char) * (filestat.st_size+1) );
     memset(buf, 0, filestat.st_size+1);
   } else {
     fprintf(stderr, "Error: File %s not found.\n", filename);
@@ -1582,7 +1938,7 @@ MC_RetrieveAgent(MCAgency_t attr) /*{{{*/
   MCAgent_t agent=NULL;
   MUTEX_LOCK(attr->mc_platform->agent_queue->lock);
   for (i = 0; i < attr->mc_platform->agent_queue->size; i++) {
-    agent = ListSearch(
+    agent = (MCAgent_t)ListSearch(
         attr->mc_platform->agent_queue->list, i);
     if (agent->agent_status == MC_AGENT_NEUTRAL) {
       break;
@@ -1653,7 +2009,7 @@ MC_SearchForService( /*{{{*/
   search->search_string = (char*)searchString;
 
   request->data = (void*)search;
-  request->command = malloc(sizeof(char) * 7);
+  request->command = (char*)malloc(sizeof(char) * 7);
   strcpy((char*)request->command, "search");
   request->data_size = sizeof(df_request_search_t);
 
@@ -1701,7 +2057,7 @@ MC_SemaphoreWait(MCAgency_t attr, int id) /*{{{*/
   return 0;
 } /*}}}*/
 
-int
+EXPORTMC int
 MC_SendCh(MCAgency_t attr, /*{{{*/
     const char *filename,
     const char *remotehost,
@@ -1711,13 +2067,111 @@ MC_SendCh(MCAgency_t attr, /*{{{*/
   return -1;
 } /*}}}*/
 
+EXPORTMC int 
+MC_SendAgent(MCAgency_t attr, /*{{{*/
+    const char *string)
+{
+  message_p message;
+  message = message_New();
+  if(
+      message_InitializeFromString
+      (
+       attr->mc_platform,
+       message,
+       string,
+       NULL,
+       0,
+       "ams"
+      )
+    )
+  {
+    message_Destroy(message);
+    return MC_ERR;
+  } else {
+    return message_queue_Add
+      (
+       attr->mc_platform->message_queue,
+       message
+      );
+  }
+} /*}}}*/
+
+EXPORTMC int
+MC_SendAgentFile(MCAgency_t attr,  /*{{{*/
+    const char *filename)
+{
+  struct stat filestat;
+  char *buf;
+  FILE *fp;
+  int ret;
+  message_p message;
+  agent_t *agent;
+  extern mc_platform_p g_mc_platform;
+  mc_platform_p platform;
+  buf = NULL;
+  filestat.st_size = 0;
+  stat(filename, &filestat);
+  if (filestat.st_size != 0 ) {
+    buf = (char*)malloc( sizeof(char) * (filestat.st_size+1) );
+    memset(buf, 0, filestat.st_size+1);
+  } else {
+    fprintf(stderr, "Error: File %s not found.\n", filename);
+    return 1;
+  }
+
+  if (attr == NULL) {
+    platform = g_mc_platform;
+  } else {
+    platform = attr->mc_platform;
+  }
+
+  fp = fopen(filename, "r");
+  fread((void*)buf, filestat.st_size, 1, fp);
+  fclose(fp);
+
+  message = message_New();
+  if( 
+      message_InitializeFromString
+      (
+       platform,
+       message,
+       buf,
+       NULL,
+       0,
+       "ams"
+      )
+    )
+  {
+    message_Destroy(message);
+  } else {
+    /* Here, we wish to fully parse the message and create a fully qualified
+     * agent, then let the agency send the agent. */
+    agent = agent_Initialize( platform, message, rand() );
+    if (agent == NULL) {
+      fprintf(stderr, "Error initializing agent. %s:%d", __FILE__, __LINE__);
+    }
+    agent->agent_status = MC_WAIT_MESSGSEND;
+    agent_queue_Add( platform->agent_queue, agent);
+    /* Set the ams to run */
+    MUTEX_LOCK(platform->ams->runflag_lock);
+    platform->ams->run = 1;
+    COND_BROADCAST(platform->ams->runflag_cond);
+    MUTEX_UNLOCK(platform->ams->runflag_lock);
+  }
+  free(buf);
+  return ret;
+} /*}}}*/
+
 EXPORTMC int 
 MC_SendAgentMigrationMessage(MCAgency_t attr, /*{{{*/
     const char *string,
     const char *hostname,
     int port)
 {
+
   message_p message;
+  printf("\nThe function MC_SendAgentMigrationMessage() is deprecated.\n\
+  Please use MC_SendAgentFile() instead.\n");
   message = message_New();
   if(
       message_InitializeFromString
@@ -1756,11 +2210,13 @@ MC_SendAgentMigrationMessageFile(MCAgency_t attr,  /*{{{*/
   agent_t *agent;
   extern mc_platform_p g_mc_platform;
   mc_platform_p platform;
+  printf("\nThe function MC_SendAgentMigrationMessage() is deprecated.\n\
+  Please use MC_SendAgentFile() instead.");
   buf = NULL;
   filestat.st_size = 0;
   stat(filename, &filestat);
   if (filestat.st_size != 0 ) {
-    buf = malloc( sizeof(char) * (filestat.st_size+1) );
+    buf = (char*)malloc( sizeof(char) * (filestat.st_size+1) );
     memset(buf, 0, filestat.st_size+1);
   } else {
     fprintf(stderr, "Error: File %s not found.\n", filename);
@@ -1820,8 +2276,8 @@ MC_SendSteerCommand(MCAgency_t attr, enum MC_SteerCommand_e cmd) /*{{{*/
   return 0;
 } /*}}}*/
 
-int
-MC_SetAgentStatus(MCAgent_t agent, int status) /*{{{*/
+EXPORTMC int
+MC_SetAgentStatus(MCAgent_t agent, enum MC_AgentStatus_e status) /*{{{*/
 {
   MUTEX_LOCK(agent->lock);
   agent->agent_status = status;
@@ -1835,7 +2291,7 @@ MC_SetAgentStatus(MCAgent_t agent, int status) /*{{{*/
   return 0;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_SetDefaultAgentStatus(/*{{{*/
     MCAgency_t agency,
     enum MC_AgentStatus_e status
@@ -1997,7 +2453,7 @@ MC_TerminateAgentWG(MCAgent_t calling_agent, MCAgent_t agent) /*{{{*/
   return status;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_MainLoop(MCAgency_t attr) 
 {
   MUTEX_LOCK( attr->mc_platform->quit_lock );
@@ -2036,7 +2492,7 @@ MC_WaitRetrieveAgent(MCAgency_t attr) /*{{{*/
   MC_WaitSignal(attr, MC_RECV_AGENT);
   MUTEX_LOCK(attr->mc_platform->agent_queue->lock);
   index = attr->mc_platform->agent_queue->size-1;
-  agent = ListSearch(
+  agent = (MCAgent_t)ListSearch(
       attr->mc_platform->agent_queue->list, index);
   MUTEX_UNLOCK(attr->mc_platform->agent_queue->lock);
   return agent;
@@ -2068,7 +2524,7 @@ MC_WaitSignal(MCAgency_t attr, int signals) /*{{{*/
  * *********************** */
 
 /* MC_AclDestroy */
-int MC_AclDestroy_chdl(void* varg)
+EXPORTMC int MC_AclDestroy_chdl(void* varg)
 {
   int retval;
   fipa_acl_message_t* acl_message;
@@ -2083,7 +2539,7 @@ int MC_AclDestroy_chdl(void* varg)
 }
 
 /* MC_AclNew */
-void* MC_AclNew_chdl(void* varg)
+EXPORTMC void* MC_AclNew_chdl(void* varg)
 {
   void* retval;
   retval = (void*)MC_AclNew();
@@ -2091,7 +2547,7 @@ void* MC_AclNew_chdl(void* varg)
 }
 
 /* MC_AclPost */
-int MC_AclPost_chdl(void* varg)
+EXPORTMC int MC_AclPost_chdl(void* varg)
 {
   int retval;
   agent_p agent;
@@ -2182,6 +2638,85 @@ MC_AclWaitRetrieve_chdl(void *varg)
 /* BEGIN Acl helper functions */
 
 /* MC_AclSetProtocol */
+EXPORTCH int 
+MC_AclGetProtocol_chdl(void* varg)
+{
+  ChInterp_t interp;
+  ChVaList_t ap;
+  struct fipa_acl_message_s* acl;
+  int retval;
+
+  Ch_VaStart(interp, ap, varg);
+  acl = Ch_VaArg(interp, ap, struct fipa_acl_message_s*);
+  retval = MC_AclGetProtocol(acl);
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
+EXPORTCH char*
+MC_AclGetConversationID_chdl(void* varg)
+{
+  ChInterp_t interp;
+  ChVaList_t ap;
+  struct fipa_acl_message_s* acl;
+  char* retval;
+
+  Ch_VaStart(interp, ap, varg);
+  acl = Ch_VaArg(interp, ap, struct fipa_acl_message_s*);
+  retval = MC_AclGetConversationID(acl);
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
+EXPORTCH int
+MC_AclGetPerformative_chdl(void* varg)
+{
+  ChInterp_t interp;
+  ChVaList_t ap;
+  struct fipa_acl_message_s* acl;
+  int retval;
+
+  Ch_VaStart(interp, ap, varg);
+  acl = Ch_VaArg(interp, ap, struct fipa_acl_message_s*);
+  retval = MC_AclGetPerformative(acl);
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
+EXPORTCH int
+MC_AclGetSender_chdl(void* varg)
+{
+  ChInterp_t interp;
+  ChVaList_t ap;
+  struct fipa_acl_message_s* acl;
+  char** name;
+  char** address;
+  int retval;
+
+  Ch_VaStart(interp, ap, varg);
+  acl = Ch_VaArg(interp, ap, struct fipa_acl_message_s*);
+  name = Ch_VaArg(interp, ap, char**);
+  address = Ch_VaArg(interp, ap, char**);
+  retval = MC_AclGetSender(acl, name, address);
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
+EXPORTCH char*
+MC_AclGetContent_chdl(void* varg)
+{
+  ChInterp_t interp;
+  ChVaList_t ap;
+  struct fipa_acl_message_s* acl;
+  char* retval;
+
+  Ch_VaStart(interp, ap, varg);
+  acl = Ch_VaArg(interp, ap, struct fipa_acl_message_s*);
+  retval = (char*)MC_AclGetContent(acl);
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
 EXPORTCH int 
 MC_AclSetProtocol_chdl(void* varg)
 {
@@ -2197,7 +2732,6 @@ MC_AclSetProtocol_chdl(void* varg)
   retval = MC_AclSetProtocol(acl, protocol);
   Ch_VaEnd(interp, ap);
   return retval;
-
 }
 
 /* MC_AclSetContentID */
@@ -2340,11 +2874,45 @@ MC_AddAgent_chdl(void *varg) /*{{{*/
   return retval;
 } /*}}}*/
 
+/* MC_AgentAddTask */
+EXPORTCH int 
+MC_AgentAddTask_chdl(void *varg) /*{{{*/
+{
+  int retval;
+
+  /* Function Args */
+  MCAgent_t agent;
+  const char* code;
+  const char* return_var_name;
+  const char* server;
+  int persistent;
+
+  ChInterp_t interp;
+  ChVaList_t ap;
+
+  Ch_VaStart(interp, ap, varg);
+
+  agent = Ch_VaArg(interp, ap, MCAgent_t);
+  code = Ch_VaArg(interp, ap, const char*);
+  return_var_name = Ch_VaArg(interp, ap, const char*);
+  server = Ch_VaArg(interp, ap, const char*);
+  persistent = Ch_VaArg(interp, ap, int);
+
+  retval= MC_AgentAddTask(
+      agent,
+      code,
+      return_var_name,
+      server,
+      persistent);
+  Ch_VaEnd(interp, ap);
+  return retval;
+} /*}}}*/
+
 /* MC_AgentVariableRetrieve */
 EXPORTCH const void*
 MC_AgentVariableRetrieve_chdl(void* varg)
 {
-  void* retval;
+  const void* retval;
   MCAgent_t agent;
   const char* var_name;
   int task_num;
@@ -2545,15 +3113,15 @@ MC_ComposeAgent_chdl(void *varg) /*{{{*/
       code,
       return_var_name,
       server,
-      NULL,
-      persistent);
+      persistent,
+      NULL);
   Ch_VaEnd(interp, ap);
   return retval;
 } /*}}}*/
 
 /* MC_ComposeAgent */
 EXPORTCH MCAgent_t 
-MC_ComposeAgentS_chdl(void *varg) /*{{{*/
+MC_ComposeAgentWithWorkgroup_chdl(void *varg) /*{{{*/
 {
   MCAgent_t retval;
 
@@ -2588,8 +3156,92 @@ MC_ComposeAgentS_chdl(void *varg) /*{{{*/
       code,
       return_var_name,
       server,
-			workgroup_code,
-      persistent);
+      persistent,
+			workgroup_code);
+  Ch_VaEnd(interp, ap);
+  return retval;
+} /*}}}*/
+
+/* MC_ComposeAgentFromFile */
+EXPORTCH MCAgent_t 
+MC_ComposeAgentFromFile_chdl(void *varg) /*{{{*/
+{
+  MCAgent_t retval;
+
+  /* Function Args */
+  const char* name;
+  const char* home;
+  const char* owner;  
+  const char* filename;
+  const char* return_var_name;
+  const char* server;
+  int persistent;
+
+  ChInterp_t interp;
+  ChVaList_t ap;
+
+  Ch_VaStart(interp, ap, varg);
+
+  name  = Ch_VaArg(interp, ap, const char*);
+  home  = Ch_VaArg(interp, ap, const char*);
+  owner = Ch_VaArg(interp, ap, const char*);
+  filename = Ch_VaArg(interp, ap, const char*);
+  return_var_name = Ch_VaArg(interp, ap, const char*);
+  server = Ch_VaArg(interp, ap, const char*);
+  persistent = Ch_VaArg(interp, ap, int);
+
+  retval= MC_ComposeAgentFromFileWithWorkgroup(
+      name,
+      home,
+      owner,
+      filename,
+      return_var_name,
+      server,
+      persistent,
+      NULL);
+  Ch_VaEnd(interp, ap);
+  return retval;
+} /*}}}*/
+
+/* MC_ComposeAgentFromFileWithWorkgroup */
+EXPORTCH MCAgent_t 
+MC_ComposeAgentFromFileWithWorkgroup_chdl(void *varg) /*{{{*/
+{
+  MCAgent_t retval;
+
+  /* Function Args */
+  const char* name;
+  const char* home;
+  const char* owner;  
+  const char* filename;
+  const char* return_var_name;
+  const char* server;
+  const char* workgroup;
+  int persistent;
+
+  ChInterp_t interp;
+  ChVaList_t ap;
+
+  Ch_VaStart(interp, ap, varg);
+
+  name  = Ch_VaArg(interp, ap, const char*);
+  home  = Ch_VaArg(interp, ap, const char*);
+  owner = Ch_VaArg(interp, ap, const char*);
+  filename = Ch_VaArg(interp, ap, const char*);
+  return_var_name = Ch_VaArg(interp, ap, const char*);
+  server = Ch_VaArg(interp, ap, const char*);
+  workgroup = Ch_VaArg(interp, ap, const char*);
+  persistent = Ch_VaArg(interp, ap, int);
+
+  retval= MC_ComposeAgentFromFileWithWorkgroup(
+      name,
+      home,
+      owner,
+      filename,
+      return_var_name,
+      server,
+      persistent,
+      workgroup);
   Ch_VaEnd(interp, ap);
   return retval;
 } /*}}}*/
@@ -3333,7 +3985,7 @@ MC_SetAgentStatus_chdl(void *varg) /*{{{*/
   Ch_VaStart(interp, ap, varg);
   agent = Ch_VaArg(interp, ap, MCAgent_t);
   status = Ch_VaArg(interp, ap, int);
-  ret = MC_SetAgentStatus(agent, status);
+  ret = MC_SetAgentStatus(agent, (enum MC_AgentStatus_e) status);
   Ch_VaEnd(interp, ap);
   return ret;
 } /*}}}*/
@@ -3355,7 +4007,7 @@ MC_SetDefaultAgentStatus_chdl(void *varg) /*{{{*/
 
   Ch_VaStart(interp, ap, varg);
   status = Ch_VaArg(interp, ap, int);
-  ret = MC_SetDefaultAgentStatus(temp_attr, status);
+  ret = (int)MC_SetDefaultAgentStatus(temp_attr, (enum MC_AgentStatus_e)status);
   Ch_VaEnd(interp, ap);
   free(temp_attr);
   return ret;
@@ -3442,7 +4094,7 @@ MC_TerminateAgentWG_chdl(void *varg) /*{{{*/
 
   Ch_VaStart(interp, ap, varg);
   calling_agent = Ch_VaArg(interp, ap, MCAgent_t);
-  agentName = Ch_VaArg(interp, ap, MCAgent_t);
+  agentName = Ch_VaArg(interp, ap, const char*);
   agent = MC_FindAgentByName(NULL, agentName);
   if (agent == NULL) {
     return MC_ERR_NOT_FOUND;
