@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 174 $ : Last Committed Revision
- * $Date: 2008-06-24 10:50:29 -0700 (Tue, 24 Jun 2008) $ : Last Committed Date */
+ * $Revision: 517 $ : Last Committed Revision
+ * $Date: 2010-06-11 12:06:47 -0700 (Fri, 11 Jun 2010) $ : Last Committed Date */
 #ifndef _WIN32
 #include <pthread.h>
 #endif
@@ -71,16 +71,16 @@ syncListNode_t *syncListFind(int id, struct syncList_s *list) { /*{{{*/
 
 struct syncList_s* syncListInit(void) /*{{{*/
 {
-    struct syncList_s* new;
-    new = (struct syncList_s*)malloc(sizeof(struct syncList_s));
-    new->lock = (RWLOCK_T*)malloc(sizeof(RWLOCK_T));
-    RWLOCK_INIT(new->lock);
+    struct syncList_s* newList;
+    newList = (struct syncList_s*)malloc(sizeof(struct syncList_s));
+    newList->lock = (RWLOCK_T*)malloc(sizeof(RWLOCK_T));
+    RWLOCK_INIT(newList->lock);
 
-    new->giant_lock = (MUTEX_T*)malloc(sizeof(MUTEX_T));
-    MUTEX_INIT(new->giant_lock);
+    newList->giant_lock = (MUTEX_T*)malloc(sizeof(MUTEX_T));
+    MUTEX_INIT(newList->giant_lock);
 
-    new->list = ListInitialize();
-    return new;
+    newList->list = ListInitialize();
+    return newList;
 } /*}}}*/
 
 int syncListAddNode(struct syncListNode_s *node, struct syncList_s *list) { /*{{{*/
