@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 453 $ : Last Committed Revision
- * $Date: 2010-05-07 14:31:08 -0700 (Fri, 07 May 2010) $ : Last Committed Date */
+ * $Revision: 527 $ : Last Committed Revision
+ * $Date: 2010-06-16 13:25:08 -0700 (Wed, 16 Jun 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -70,8 +70,8 @@ message_New(void)
   message->connect_id = 0;
   message->message_id = 0;
   message->isHTTP = 0;
-  message->message_type = 0;
-  message->http_type = 0;
+  message->message_type = (enum message_type_e)0;
+  message->http_type = (enum http_performative_e)0;
   message->xml_root = NULL;
   message->xml_payload = NULL;
   message->message_body = NULL;
@@ -234,7 +234,7 @@ message_InitializeFromConnection(
 		n = recv(connection->clientfd, (void*)buffer, (size_t) sizeof(char)*SOCKET_INPUT_SIZE, MSG_WAITALL);
 #else
     n = recvfrom(connection->clientfd,
-        (void *) buffer,
+        buffer,
         (size_t) sizeof(char)*SOCKET_INPUT_SIZE,
         0,
         (struct sockaddr *) 0,
@@ -252,7 +252,7 @@ message_InitializeFromConnection(
       buffer[n] = '\0';
       i++;
       strcat(message_string, buffer);
-      message_string = realloc
+      message_string = (char*)realloc
         (
          message_string, 
          sizeof(char) * (SOCKET_INPUT_SIZE+1) * i
@@ -290,7 +290,7 @@ int http_to_hostport(const char* http_str, char** host, int* port, char** target
    * host: somehost.com
    * port: 5050
    * target: acc */
-  char* tmp;
+  const char* tmp;
   if(strncmp(http_str, "http://", 7)) {
     return MC_ERR_PARSE;
   }
@@ -346,16 +346,22 @@ message_InitializeFromString(
 
   message->update_name = NULL;
 
-  destination = malloc(sizeof(char)*(strlen(destination_host) + 10));
-  CHECK_NULL(destination,
-      mc_platform->err = MC_ERR_MEMORY;
-      return MC_ERR_MEMORY; );
-  sprintf(destination, "%s:%d", 
-      destination_host,
-      destination_port
-      );
+  if(destination_host != NULL) {
+    destination = (char*)malloc(sizeof(char)*(strlen(destination_host) + 10));
+    CHECK_NULL(destination,
+        mc_platform->err = MC_ERR_MEMORY;
+        return MC_ERR_MEMORY; );
+    sprintf(destination, "%s:%d", 
+        destination_host,
+        destination_port
+        );
+    message->to_address = destination;
+  } else {
+    message->to_address = NULL;
+  }
 
-  message->to_address = destination;
+  /* DEBUG */
+  //message->to_address = destination;
   message->from_address = (char*)malloc(
       sizeof(char) * (strlen(mc_platform->hostname)+10));
   sprintf(message->from_address,
@@ -387,7 +393,6 @@ message_InitializeFromString(
   if (message_xml_parse(message)) {
     fprintf(stderr, "Error parsing message at %s:%d.\n",
         __FILE__, __LINE__);
-    message_Destroy(message);
     return MC_ERR_PARSE;
   }
 
@@ -440,16 +445,13 @@ auth_rece_send_msg(int sockfd, char *hostname, char *message, char *privkey, cha
   unsigned char passphrase[35], aes_key[35];
   int nonce, mode;
   char privatekey[1215];
-  char peer_pubkey[300], ch;
+  char peer_pubkey[300];
   char plaintext[135];
-  FILE *fd, *tempfd;
-  char buffer[10];
-  struct stat stbuf;
+  FILE *fd;
 
   /* temporary files  */
   char *infile; //= "msg";     // message
   char *outfile; //= "en-msg"; // encrypted message
-  char *tempfile;
   memset(passphrase ,  '\0', 35);
   memset(privatekey ,  '\0', 1215);
   memset(plaintext  ,  '\0', 135);
@@ -463,7 +465,7 @@ auth_rece_send_msg(int sockfd, char *hostname, char *message, char *privkey, cha
   infile = (char *)malloc(sizeof(char)*20);    
   strcpy(infile, "msgXXXXXX");
   fd1 = mkstemp(infile);
-  if (fd == -1) {
+  if (fd1 == -1) {
      fprintf(stderr, "Could not create temporary file:%s. %s:%d\n",
           infile, __FILE__, __LINE__ );
      exit(EXIT_FAILURE);
@@ -584,6 +586,7 @@ int message_Send(mc_platform_t* mc_platform, message_p message, char *privatekey
 	/* DEBUG */
 	THREAD_CREATE(&msg_thread, message_send_Thread, arg);
 	THREAD_DETACH(msg_thread);
+  return 0;
 }
 
 
@@ -609,17 +612,18 @@ message_send_Thread( LPVOID arg )
 	char* privatekey;
   char *buffer;
   mtp_http_t* mtp_http;
-  int n, ret;
+  int n;
 #ifndef _WIN32
   int skt;
   struct sockaddr_in sktin;
   struct addrinfo* myaddrinfo;
+  struct addrinfo hint_addrinfo;
 #else
   SOCKET skt;
   SOCKADDR_IN sktin;
-#endif
   struct hostent host;
-  struct hostent *host_result;
+  struct hostent* host_result;
+#endif
   char *buf;
   char *hostname;
 #ifndef _WIN32
@@ -633,9 +637,6 @@ message_send_Thread( LPVOID arg )
 
 	dynstring_t* message_string;
 
-	char hostbuf[80];
-	int hostbuf_len = 80;
-
 	mc_platform = ((message_send_arg_t*)arg)->mc_platform;
 	message = ((message_send_arg_t*)arg)->message;
 	privatekey = ((message_send_arg_t*)arg)->privatekey;
@@ -669,7 +670,9 @@ message_send_Thread( LPVOID arg )
 
 #ifndef _WIN32
   //if(gethostbyname_r(hostname, &host, hostbuf, hostbuf_len, &host_result, &errnum)) 
-  if( (errnum = getaddrinfo(hostname, NULL, NULL, &myaddrinfo)) == 0 )
+  memset(&hint_addrinfo, 0, sizeof(hint_addrinfo));
+  hint_addrinfo.ai_family = AF_INET;
+  if( (errnum = getaddrinfo(hostname, NULL, &hint_addrinfo, &myaddrinfo)) == 0 )
 #else
   if(host_result = gethostbyname(hostname)) 
 #endif
@@ -743,7 +746,7 @@ message_send_Thread( LPVOID arg )
     //printf("successfully authenticate and send \n");        
 #else
     if(send(skt, message->message_body, strlen(message->message_body), 0) < 0) {
-      perror(send);
+      perror("send");
       printf("cannot write to socket %s:%d\n", __FILE__, __LINE__);
     }
     else
@@ -766,7 +769,7 @@ message_send_Thread( LPVOID arg )
 				n = recv(skt, (void*)buffer, (size_t) sizeof(char)*SOCKET_INPUT_SIZE, MSG_WAITALL);
 #else
 				n = recvfrom(skt,
-						(void *) buffer,
+						buffer,
 						(size_t) sizeof(char)*SOCKET_INPUT_SIZE,
 						0,
 						(struct sockaddr *) 0,
@@ -812,4 +815,4 @@ message_send_Thread( LPVOID arg )
   MSG_THREAD_EXIT();
 }
 
-#undef MSG_THREAD_EXIT()
+#undef MSG_THREAD_EXIT
