@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 402 $ : Last Committed Revision
- * $Date: 2009-08-23 13:44:39 -0700 (Sun, 23 Aug 2009) $ : Last Committed Date */
+ * $Revision: 517 $ : Last Committed Revision
+ * $Date: 2010-06-11 12:06:47 -0700 (Fri, 11 Jun 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -587,7 +587,7 @@ agent_xml_compose__create_row_nodes
     buf = (char*)malloc(sizeof(char) * 20);
     CH_DATATYPE_SIZE(type, size);
 
-    varstring = malloc(
+    varstring = (char*)malloc(
         (sizeof(char)*64) * *extent);
     varstring[0] = '\0';
     for(i = 0; i < *extent; i++) {
